// Author:       Andreas F. Borchert
// Description:  maintains a list of friends in sorted
//               order using binary trees

#include <iostream>
#include "ListOfFriends.h"

using namespace std;

struct Node {
   struct Node* left;
   struct Node* right;
   Friend f;
   Node(const Friend& newFriend);
   Node(const Node* const& node);
   ~Node();
}; // struct Node

Node::Node(const Friend& newFriend) :
   left(0), right(0), f(newFriend) {
} // Node::Node

Node::Node(const Node* const& node) :
   left(0), right(0), f(node->f) {
   if (node->left) {
      left = new Node(node->left);
   }
   if (node->right) {
      right = new Node(node->right);
   }
} // Node::Node

Node::~Node() {
   if (left) {
      delete left;
   }
   if (right) {
      delete right;
   }
} // Node::~Node

ListOfFriends::ListOfFriends() :
   root(0) {
} // ListOfFriends::ListOfFriends

ListOfFriends::ListOfFriends(const ListOfFriends& list) :
   root(0) {
   Node* r(list.root);
   if (r) {
      root = new Node (r);
   }
} // ListOfFriends::ListOfFriends

ListOfFriends::~ListOfFriends() {
   if (root) {
      delete root;
   }
} // ListOfFriends::~ListOfFriends

ListOfFriends& ListOfFriends::operator=(const ListOfFriends& list) {
   if (this != &list) { // protect against self-assignment
      if (root) {
         delete root;
      }
      if (list.root) {
         root = new Node (list.root);
      } else {
         root = 0;
      }
   }
   return *this;
} // ListOfFriends::operator=

void ListOfFriends::addto(Node*& p, Node* newNode) {
   if (p) {
      if (newNode->f.get_name() < p->f.get_name()) {
         addto(p->left, newNode);
      } else {
         addto(p->right, newNode);
      }
   } else {
      p = newNode;
   }
} // ListOfFriends::addto

void ListOfFriends::add(const Friend& f) {
   Node* node( new Node(f) );
   addto(root, node);
} // ListOfFriends::add

void ListOfFriends::visit(const Node* const p) {
   if (p) {
      visit(p->left);
      cout << p->f.get_name() << ": " << p->f.get_info() << endl;
      visit(p->right);
   }
} // ListOfFriends::visit

void ListOfFriends::print() {
   visit(root);
} // ListOfFriends::print
